import re
import traceback
from pathlib import Path

__all__ = ['get_project_root']


def get_project_root():
    stack = reversed(traceback.extract_stack())

    for frame in stack:
        try:
            path = Path(frame.filename).resolve()
        except OSError:
            # some frames have names that cannot be treated as file paths
            continue

        # packages installed using pip are stored in the 'site-packages'
        # if from_root is called from a package, we can quickly find the root directory
        posix_like = path.as_posix()
        if 'site-packages' in posix_like:
            root_path = Path(re.findall(r'.*site-packages/.*?/', posix_like)[0])
            # but we ignore 'from_root' package
            if root_path.name != 'from_root':
                return root_path

        while path.parents:
            if (
                (path / '.git').exists()
                or
                (path / '.project-root').exists()
            ):
                return path

            path = path.parent

    # TODO(ekon): put more details
    raise FileNotFoundError(
        f'There is neither ".git" directory nor ".project-root" file, '
        f'cannot detect root folder'
    )


if __name__ == '__main__':
    print(get_project_root())



from pathlib import Path
from typing import Iterable, Union

from from_root.get_project_dir import get_project_root

__all__ = ['from_root']


def from_root(
    *args: Iterable[str],
    create_parent_dirs=False,
    pathlib: bool = False,
) -> Union[str, Path]:
    """
    :param args:
    >>> from_root('dir1', 'dir2', 'dir3')
    <ROOT_DIR>/a/b/c
    :param create_parent_dirs:
        if the flag is set to True, all non-existing directories after <ROOT_DIR> will be created
    :param pathlib: whether or not to return `pathlib.Path` instead of `str`
    :return: `str` or `pathlib.Path`
    """
    args = map(str, args)
    path = get_project_root().joinpath(*args)

    if create_parent_dirs:
        _create_if_does_not_exist(path)

    if pathlib:
        path = Path(path)

    return path


def _create_if_does_not_exist(path: Path):
    if path.suffix:
        path = path.parent
    if not path.is_dir():
        path.mkdir(parents=True)



import setuptools

with open('README.md', 'r') as file:
    long_description = file.read()

setuptools.setup(
    name='from-root',
    version='0.0.1',
    author='Eduard Konanau',
    author_email='aduard.kononov@gmail.com',
    description='Forget about working directory errors',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/EduardKononov/from-root',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
